#include "xtmrctr_l.h"
#include "xintc_l.h"
#include "xparameters.h"

/* lwIP Includes */
#include "netif/xemacif.h"
#include "lwip/tcp.h"
#include "lwip/memp.h"
#include "netif/etharp.h"

#define LWIP_TIMER_CYCLES (XPAR_CPU_CORE_CLOCK_FREQ_HZ / 1000 \
                           * TCP_TMR_INTERVAL )

// Upper 6 bytes of MAC - Xilinx Ethernet OUI = 00-00-00
#define XILINX_MAC_OUI0 0x00
#define XILINX_MAC_OUI1 0x00
#define XILINX_MAC_OUI2 0x00

static void show_dotted_decimal( char * address_array);
static void show_dashed_hex( int bytes, char *address_array);

// Static Global Variables
static u8_t my_timer = 0;
static int waiting_for_timer = 1;

// External Global Variables

/* defined in lwip/src/core/tcp.c */
extern u32_t tcp_ticks; 
/* defined in EDK generated xemacif_g.c file */
extern XEmacIf_Config XEmacIf_ConfigTable[]; 


/*-------------------------------iinterrupt routine-----------------------------*/	
void mytimer_int_handler (void* baseaddr_p) {

   unsigned int csr;
	//xil_printf(".");

  /* Read timer 0 CSR to see if it raised the interrupt */
  csr = XTmrCtr_mGetControlStatusReg(XPAR_OPB_TIMER_1_BASEADDR, 0);
  
  if (csr & XTC_CSR_INT_OCCURED_MASK) {

     //xil_printf("Timer Handler ...\n\t");
     waiting_for_timer = 0;
     
     /* Clear the timer interrupt */
     XTmrCtr_mSetControlStatusReg(XPAR_OPB_TIMER_1_BASEADDR, 0, csr);
	  //clear interrupt at interrupt controller
	  XIntc_mAckIntr(XPAR_OPB_TIMER_1_BASEADDR, XPAR_OPB_TIMER_1_INTERRUPT_MASK); 
  } 
}

/*------------------------------------------------------------------------------*/	

/*---------------------------------------------------------------------------*/
// show dotted decimal prints a dotted decimal address to the UART           */
/*---------------------------------------------------------------------------*/
static void show_dotted_decimal( char *address_array)
{
   int bb;
   unsigned char temp;

   for(bb=0;bb<4;bb++)
   {
      temp = address_array[bb];
      if(bb!=0) xil_printf(".");
      xil_printf("%d", temp);      
   }
}

/*---------------------------------------------------------------------------*/
/* show dashed hex prints a dashed hex address to the UART                   */
/*---------------------------------------------------------------------------*/
static void show_dashed_hex( int bytes, char *address_array)
{
   //Assumes the caller passes the correct number of bytes
   int bb;

   for(bb=0;bb<bytes;bb++)
   {
      if(bb !=0) xil_printf("-");
      xil_printf("%02X", (int) address_array[bb]);
   }
}

/*---------------------------------------------------------------------------*/
/* my_tmr - Called Periodically to dispatch TCP and ARP timers               */
/*---------------------------------------------------------------------------*/
void my_tmr(void)
{
	//xil_printf(".");
   ++my_timer;

   if(my_timer == 10) {
      my_timer = 0;
   }
   if(my_timer & 1) {
      /* Call tcp_fasttmr() every 2 ms, i.e.,
       * every other timer my_tmr() is called. */
      tcp_fasttmr();
   }
   if(my_timer == 0 || my_timer == 5) {
      /* Call tcp_slowtmr() every 5 ms, i.e.,
       * every fifth timer my_tmr() is called. */
      tcp_slowtmr();
      if (tcp_ticks%2000 == 0)
         /* Call etharp_tmr() every 20th call to tcp_slowtmr().
          * tcp_ticks is a global var defined in core/tcp.c */
         etharp_tmr();
   }
}

/*---------------------------------------------------------------------------*/
/* print_app_header - prints the legal header                                */
/*---------------------------------------------------------------------------*/
void print_app_header()
{
   xil_printf("\n\n\n\n\n\n\n\n\r\n");
   xil_printf("####################################################################\r\n");
   xil_printf("# Xilinx TCP/IP Demo Application (ECHO  Server)                    #\r\n");
   xil_printf("#                                                                  #\r\n");
   xil_printf("# XILINX IS PROVIDING THIS DESIGN, CODE, OR INFORMATION \"AS IS\"    #\r\n");
   xil_printf("# SOLELY FOR USE IN DEVELOPING PROGRAMS AND SOLUTIONS FOR          #\r\n");
   xil_printf("# XILINX DEVICES.                                                  #\r\n");
   xil_printf("#                                                                  #\r\n");
   xil_printf("# (c) Copyright 2003, 2003 Xilinx, Inc.                            #\r\n");
   xil_printf("# All rights reserved.                                             #\r\n");
   xil_printf("#                                                                  #\r\n");
   xil_printf("####################################################################\r\n");
}

/*---------------------------------------------------------------------------*/

/*-------------------------------main function----------------------------------*/	
int main(){
	void *callBack;

	xil_printf("\n\n\r Timer and interrupt test \n\r");

   struct tcp_pcb *gddpp_pcb;
   struct ip_addr ipaddr, netmask, gw;
   struct netif *default_netif;//, *tmp_netif;
   char menu_select = 0;

   char low_mac[3] = {0x00,0x22,0x38};
   char fullmac[6] = {XILINX_MAC_OUI0, XILINX_MAC_OUI1, XILINX_MAC_OUI2,
                      low_mac[0], low_mac[2], low_mac[3]};
   char ip[4] =      {192,168,201,165};
   char subnet[4] =  {255,255,255,0};
   char gateway[4] = {192,168,201,20};
   int  app_running = 0;
   unsigned int init_wait = 15000000;

   /*-------------------------------
    * Enable microblaze interrupts
    *-------------------------------*/
     microblaze_enable_interrupts();
   
   /*-------------------------------------------------------------------------
    * Timer and Intc Init
    *
    * Set the Timer to interrupt for every 100ms
    *------------------------------------------------------------------------*/
   
   /* set the number of cycles the timer counts before interrupting */
   /* 67 Mhz clock => 15 ns for 1 clk tick. For 1ms, 66 667 clk ticks need to elapse  */
   XTmrCtr_mSetLoadReg(XPAR_OPB_TIMER_1_BASEADDR, 0, 66667);

   /* reset the timers, and clear interrupts */
   XTmrCtr_mSetControlStatusReg(XPAR_OPB_TIMER_1_BASEADDR, 0, XTC_CSR_INT_OCCURED_MASK | XTC_CSR_LOAD_MASK );
   
   /* start the timers */
   XTmrCtr_mSetControlStatusReg(XPAR_OPB_TIMER_1_BASEADDR, 0, XTC_CSR_ENABLE_TMR_MASK | XTC_CSR_ENABLE_INT_MASK | XTC_CSR_AUTO_RELOAD_MASK | XTC_CSR_DOWN_COUNT_MASK);

	//register my handler
	XIntc_RegisterHandler(XPAR_OPB_INTC_0_BASEADDR,
								XPAR_OPB_INTC_0_OPB_TIMER_1_INTERRUPT_INTR,
								mytimer_int_handler,
								callBack);

	//Enable timer interrupts
	XIntc_mEnableIntr (XPAR_OPB_INTC_0_BASEADDR, XPAR_OPB_TIMER_1_INTERRUPT_MASK);

	//Enable interrupt controller output to CPU interrupt
	XIntc_mMasterEnable(XPAR_OPB_INTC_0_BASEADDR);

   /*-----------------------------------------------------------------------*/
   /* Do LWIP System Inits                                                  */
   /*-----------------------------------------------------------------------*/
#ifdef STATS
   stats_init();
#endif /* STATS */
   xil_printf("Initializing Memory Structures.");
   sys_init();
   mem_init();
   xil_printf(".");
   memp_init();
   xil_printf(".");
   pbuf_init();
   xil_printf(" done.\r\n");

   /*-----------------------------------------------------------------------*/
   /* Initial Header and Menus.  Do this before the netif_init() so we can  */
   /* change the MAC Address and IP addresses if needed                     */
   /*-----------------------------------------------------------------------*/
   while(init_wait--);
   print_app_header();

   fullmac[0] = XILINX_MAC_OUI0;
   fullmac[1] = XILINX_MAC_OUI1;
   fullmac[2] = XILINX_MAC_OUI2;
   fullmac[3] = low_mac[0];
   fullmac[4] = low_mac[1];
   fullmac[5] = low_mac[2];

   
   /*-----------------------------------------------------------------------*/
   /* Set host addresses                                                    */
   /*-----------------------------------------------------------------------*/
   xemacif_setmac(0, (u8_t *) fullmac);                        //Set MAC
   IP4_ADDR(&gw, gateway[0],gateway[1],gateway[2],gateway[3]); //Set gateway
   IP4_ADDR(&ipaddr, ip[0],ip[1],ip[2],ip[3]);                 //Set ip
   IP4_ADDR(&netmask,subnet[0],subnet[1],subnet[2],subnet[3]); //Set subnet msk

   /*-----------------------------------------------------------------------*/
   /* Show some host boot stuff and parameters                              */
   /*-----------------------------------------------------------------------*/
   xil_printf("\r\nStarting Network Interface...\r\n");
   xil_printf("  MAC Address: "); show_dashed_hex(6, fullmac);  xil_printf("\r\n");
   xil_printf("  IP Address: ");  show_dotted_decimal(ip);      xil_printf("\r\n");
   xil_printf("  Subnet Mask: "); show_dotted_decimal(subnet);  xil_printf("\r\n");
   xil_printf("  Gateway IP: ");  show_dotted_decimal(gateway); xil_printf("\r\n");
   xil_printf("  Echo Port: 7 \r\n");
      
   /*-----------------------------------------------------------------------*/
   /* Initialize netif                                                      */
   /*-----------------------------------------------------------------------*/
   netif_init();

   /*-----------------------------------------------------------------------*/
   /* Initialize TCP Stack                                                  */
   /*-----------------------------------------------------------------------*/
   tcp_init();

   /*-----------------------------------------------------------------------*/
   /* Set up the lwIP network interface...                                  */
   /*-----------------------------------------------------------------------*/
	// Set up the lwIP network interface
	// Allocate and configure the server's netif
	default_netif = mem_malloc(sizeof(struct netif));
	if(default_netif == NULL)
	{
	  xil_printf("ERROR: netif_add(): Out of memory for default netif\n\r");
	  return;
	}

   default_netif = netif_add(default_netif, //tmp_netif,
	                          &ipaddr, 
                             &netmask,
                             &gw,
                             &XEmacIf_ConfigTable[0],
                             xemacif_init,
                             ip_input
                            );

	   netif_set_default(default_netif);

   /*-----------------------------------------------------------------------*/
   /* create new tcp pcb and start applications                             */
   /*-----------------------------------------------------------------------*/
   
   // Start the Server
   xil_printf("Echo Server Running ... "); xil_printf("\r\n");
   echo_init();
   app_running = 1;

   while (1) {
      while (waiting_for_timer) {
         xemacif_input(default_netif);
      }
      my_tmr(); 
      waiting_for_timer = 1;
   }
   return (1);


}
/*------------------------------------------------------------------------------*/	
